/*
 * command.cc  - irx command class
 *
 * Copyright (C) 1998 Alan Penner        (apenner@andrew.cmu.edu)
 *                    Ryan de Jonckheere (rjd@andrew.cmu.edu)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *
 */

#include <X11/Xlib.h>
#include <iostream.h>
#include <stdlib.h>
#include "command.h"
#include "winlist.h"

extern void getwindows(winlist& w, Display* d, Window w1);
extern string cppstring( char* a );

command::command(action_type t, string k, nfa* n )
{
  string let;
  type = t;
  next = NULL;
  window_name = n;
  comm = k;
}

void command::add_event(action_type t, string k, nfa* n )
{
  command* tmp;
  for(tmp = this; tmp -> next; tmp = tmp -> next);
  tmp -> next = new command(t, k, n);
}

void command::do_event( winlist& windows, Display* disp )
{
  char *window_n;
  winlist_node* tmp;
  command* tmp1;
  bool found_window = false;
  Window w;

  if (type == SYSTEM_ACTION)
    {
      for (tmp = windows.head; tmp; tmp = tmp -> next)
	{
	  if ((XFetchName(disp, tmp -> w, &window_n))
	      && (window_name->accept_string( cppstring(window_n))))
	    found_window = true;
	}
      if (!found_window)
	send( w, disp, comm, type);
    }
  else
    for (tmp = windows.head; tmp; tmp = tmp -> next)
      if (XFetchName(disp, tmp -> w, &window_n))
	if (window_name->accept_string( cppstring(window_n)) )
	  for (tmp1 = this; tmp1; tmp1 = tmp1 -> next)
	    if (tmp1 -> type != SYSTEM_ACTION)
		send( tmp -> w, disp, tmp1 -> comm, tmp1 -> type);

  delete [] window_n;
  
}


void command::send( Window w , Display* disp, string a, action_type t )
{
  XEvent event;
  if (t == KEY_ACTION)
    {
      a;
      memset(&event, 0, sizeof(event));
      event.xkey.type        = KeyPress;
      event.xkey.root        = RootWindow( disp, 0 );
      event.xkey.window      = w;
      event.xkey.subwindow   = w;
      event.xkey.same_screen = True;
      event.xkey.time        = CurrentTime;
      event.xkey.state       = 0;
      event.xkey.keycode =  XKeysymToKeycode(disp, XStringToKeysym(a.c_str()));

      XSendEvent(disp, w, False, 0, &event);
      
      event.xkey.type        = KeyRelease;
      event.xkey.root        = RootWindow( disp, 0 );
      event.xkey.window      = w;
      event.xkey.subwindow   = w;
      event.xkey.same_screen = True;
      event.xkey.time        = CurrentTime;
      event.xkey.state       = 0;
      event.xkey.keycode =  XKeysymToKeycode(disp, XStringToKeysym(a.c_str()));
	
      XSendEvent(disp, w, False, 0, &event);
    }
  if (t == SYSTEM_ACTION || t == EXEC_ACTION )
    system(a.c_str());

  if (t == BUTTON_ACTION)
    {
      int i=0,x,y;
      unsigned int button;
      string but="",xs="",ys="";
      
      while (i < a.length() && a[i] != ' ')
	but = but + a[i++];
      i++;
      while( i < a.length() && a[i] != ' ')
	xs = xs + a[i++];
      i++;
      while( i < a.length())
	ys = ys + a[i++];

      button = atoi( but.c_str());
      x = atoi( xs.c_str());
      y = atoi( ys.c_str());

      XWindowAttributes attrib;
      XGetWindowAttributes(disp, w, &attrib);

      XWarpPointer( disp, None, w, 0,0,0,0, x,y);
      memset(&event, 0, sizeof(event));      
      event.xbutton.type        = ButtonPress;
      event.xbutton.send_event  = False;
      event.xbutton.display     = disp;
      event.xbutton.root        = RootWindow(disp,0);
      event.xbutton.window      = w;
      event.xbutton.subwindow   = w;
      event.xbutton.same_screen = True;
      event.xbutton.state       = 0;
      event.xbutton.button      = button;
      event.xbutton.x           = x;
      event.xbutton.y           = y;
      event.xbutton.x_root      = attrib.x + x;
      event.xbutton.y_root      = attrib.y + y;
      event.xbutton.time        = CurrentTime;
      
      XSendEvent(disp, w, True, ButtonPressMask|ButtonReleaseMask, &event);
      
      event.xbutton.type        = ButtonRelease;
      event.xbutton.send_event  = True;
      event.xbutton.display     = disp;
      event.xbutton.root        = RootWindow(disp,0);
      event.xbutton.window      = w;
      event.xbutton.subwindow   = w;
      event.xbutton.same_screen = True;
      event.xbutton.state       = 0;
      event.xbutton.button      = button;
      event.xbutton.x           = x;
      event.xbutton.y           = y;
      event.xbutton.x_root      = attrib.x + x;
      event.xbutton.y_root      = attrib.y + y;
      event.xbutton.time        = CurrentTime;
      
      XSendEvent(disp, w, True, ButtonPressMask|ButtonReleaseMask, &event);
    }
}
