/*
 * conn.cc - irx socket connection class
 *
 * Copyright (C) 1998 Alan Penner        (apenner@andrew.cmu.edu)
 *                    Ryan de Jonckheere (rjd@andrew.cmu.edu)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *
 */

#include "conn.h"

conn::conn()
{
  sname = DEFAULTLIRCSERVER;
  memset((char*) &serv_addr, 0, sizeof(serv_addr));
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_port = htons(LIRC_PORT);
}

int conn::Config(int argc, char** argv)
{
  ifstream filein;
  string buffer;
  int i;

  for(i = 1; i < argc; i++)
    {
      if(!strcmp(argv[i], "-f"))
	{
	  filein.open(argv[++i], ios::in);
	  if(!filein)
	    return false;

	  while(filein >> buffer)
	    {
	      // cerr << buffer << endl;
	      if(buffer == "LircServer")
		{
		  filein >> sname;
		}
	    }
	}
    }
  // cerr << "Loop Exited!" << endl;
  return 1;
}

int conn::Open()
{
  serv_addr.sin_addr.s_addr = inet_addr(sname.c_str());
  if(serv_addr.sin_addr.s_addr == (int) -1)
    {
      if( !(he = gethostbyname(sname.c_str())) )
	die("hostname lookup failure\n");
      else
	serv_addr.sin_addr.s_addr = ((in_addr*)(he->h_addr))->s_addr;
    }
  if ( (sockfd = socket(AF_INET, SOCK_STREAM, 0)) < 0 )
    die("Could not create socket\n");
  if( connect(sockfd, (sockaddr*)&serv_addr, sizeof(serv_addr)) < 0 )
    die("Could not connect");

  return 1;
}

int conn::Read(unsigned char in[])
{
  if(read(sockfd, in, 8) < 8)
    return -1;
  else
    return 1;
}

int conn::Close()
{
  return close(sockfd);
}
