/*
 * main.cc  - irx main program
 *
 * Copyright (C) 1998 Alan Penner        (apenner@andrew.cmu.edu)
 *                    Ryan de Jonckheere (rjd@andrew.cmu.edu)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *
 */

#include <X11/Xlib.h>
#include <stdlib.h>
#include <stdio.h>
#include <string>
#include <string.h>
#include "command.h"
#include "conn.h"
#include "nfa.h"
#include "utils.h"

const char usage[] = "usage: ";
int bigerr = 0;

// prototype
int Config(int argc, char** argv, nfa& commands, conn& connection);
int readFile(const char* filename, nfa& commands, conn& connection);
int irx_errorhandle(Display* disp, XErrorEvent* err_event);

void die(const char *fault)
{
  cerr << fault << endl;
  exit(1);
}

int main(int argc, char* argv[])
{
  nfa commands;
  conn connection;
  int ok;
  unsigned char in[8];
  int out;
  
  if(!Config(argc, argv, commands, connection))
    die(usage);

  if(!connection.Open())
    die("Could not open connection to lirc server\n");

  XSetErrorHandler(irx_errorhandle);
  commands.take_input( connection );
  
  connection.Close();
}

int Config(int argc, char** argv, nfa& commands, conn& connection)
{
  char path[30];
  const char *home=getenv("HOME");
  const char *file=".irxrc";

  strncpy(path,home,30);
  strncat(path,"/",30);
  strncat(path,file,30);

  int i;
  bool found = false;
  char filename[ strlen( home ) + 7 ];
  
  for(i = 0; i < argc; i++)
    {
      if(!strcmp(argv[i], "-f"))
	{                             // increment i BEFORE we take argv[i]
	  if(readFile(argv[++i], commands, connection) == FAIL)
	    die("Could not open file");
	  found = true;
	}
    }

  if (!found)
    if(readFile(path, commands, connection) == FAIL)
      die(".irxrc does not exist");
  
  return SUCCESS;
  
}

void make_nfa( string w, nfa*& window_name )
{
  string let;
  window_name = new nfa;
  for (int i = 0; i < w.length(); i++)
    {
      switch (w[i])
	{
	case '*':
	  let = ANY_LETTER;
	  *window_name = *window_name & * (nfa(let));
	  break;
	case '?':
	  let = ANY_LETTER;
	  *window_name = *window_name & (nfa(let));
	  break;
	default:
	  let = w[i];
	  *window_name = *window_name & nfa(let);
	}
    }  
}


action_type getaction( string k )
{
  if (k == "<key>")
    return KEY_ACTION;
  if (k == "<system>")
    return SYSTEM_ACTION;
  if (k == "<exec>")
    return EXEC_ACTION;
  if (k == "<button>")
    return BUTTON_ACTION;
}

int readFile(const char* filename, nfa& commands, conn& connection)
{
  ifstream filein;
  nfa current;
  nfa* window;
  filein.open(filename, ios::in);
  if(!filein)
    return FAIL;

  //  current.print();
  string window_name;
  string event_type;
  string ircode;
  string buffer;
  string k;
  
  while (gettoken ( filein, buffer ) )
    {
      if (buffer == "Window")
	{
	  if (gettoken ( filein, window_name) )
	    make_nfa( window_name, window );
	  else
	    die ("parse error reading window name");
	  
	  while (gettoken (filein, ircode) && ircode != "EndWindow" )
	    {
	      if (ircode != ":")
		current = current & nfa(strtouq(ircode.c_str(), (char**)NULL, 16));
	      while (gettoken ( filein, ircode ) && ircode != ":")
		current = current & nfa(strtouq(ircode.c_str(), (char**)NULL, 16));

	      while (gettoken ( filein, event_type ) && event_type != ":")
		{
		  gettoken ( filein, k );
		  if (current.finish -> what_to_do)
		    current.finish -> what_to_do ->
		      add_event(getaction(event_type), k, window );
		  else
		    current.finish->what_to_do =
		      new command(getaction(event_type), k, window );
		}
	      commands = commands | current;
	      current.kill(1);
	    }
	}
    }
  
  commands.start -> add_trans(commands.start, ANY_LETTER);

  return SUCCESS;
}


int irx_errorhandle(Display* disp, XErrorEvent* err_event)
{
  char buf[256];
  XGetErrorText(disp, err_event->error_code, buf, 255);
  // cerr << "error handler, BadWindow = " << BadWindow << " error = " << (int) err_event->error_code << buf << endl;
  if((int) err_event->error_code == BadWindow)
    bigerr = BadWindow;
  return err_event->error_code;
}
