/*
 * nfa.cc - irx regexp matching class
 *
 * Copyright (C) 1998 Alan Penner        (apenner@andrew.cmu.edu)
 *                    Ryan de Jonckheere (rjd@andrew.cmu.edu)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *
 */

#include<iostream.h>
#include<string>
#include<string.h>
#include<stdio.h>
#include<stdlib.h>
#include "nfa.h"
#include "conn.h"
#include "command.h"
#include "winlist.h"


void state::add_state( char let )
{
  state* newstate = new state;
  newstate -> next = next;  
  next = newstate;
  add_trans( next, let );
}

void state::add_trans( state* to, char let )
{
  transition* newtrans = new transition( to, let );
  newtrans -> next = trans;
  trans = newtrans;
}

slist_node* state::do_trans( char let )
{
  slist_node* retval = NULL;
  slist_node* l = NULL;
  for (transition* tmp = trans; tmp; tmp = tmp -> next)
    if ( tmp -> input == let || tmp -> input == ANY_LETTER )
      {
	l = new slist_node(tmp -> to);
	l -> next = retval;
	retval = l;
      }

  return retval;
}

slist_node* state::do_trans( ullong let )
{
  slist_node* retval = NULL;
  slist_node* l = NULL;
  for (transition* tmp = trans; tmp; tmp = tmp -> next)
    if ( tmp -> input == let || tmp -> input == ANY_LETTER )
      {
	l = new slist_node(tmp -> to);
	l -> next = retval;
	retval = l;
      }

  return retval;
}

void nfa::index() const
{
  int i = 0;
  for (state* tmp = start; tmp; tmp = tmp -> next)
    tmp -> index = i++;
}

void nfa::print() const
{
  index();
  for (state* tmp = start; tmp; tmp = tmp -> next)
    {
      cout << tmp -> index << ": ";
      for (transition* tmp1 = tmp -> trans; tmp1; tmp1 = tmp1 -> next)
	{
	  if (tmp1 -> input == EPSILON)
	    cout << "EPSILON->" << tmp1 -> to -> index << " ";
	  else
	    cout << tmp1->input << "->" << tmp1 -> to -> index << " ";
	}
      if (tmp -> accept)
	cout << "ACCEPT";
      cout << endl;
    }
}

nfa::nfa( ullong ircode )
{
  start = new state;
  start -> add_state( ircode );
  state* tmp = start -> next;
  finish = tmp;
  finish -> accept = true;
}

void state::add_state( ullong ircode )
{
  state* newstate = new state;
  newstate -> next = next;  
  next = newstate;
  add_trans( next, ircode );
}

void state::add_trans( state* to, ullong ircode )
{
  transition* newtrans = new transition( to, ircode );
  newtrans -> next = trans;
  trans = newtrans;
}

nfa::nfa( string word )
{
  start = new state;
  start -> add_state( EPSILON );
  state* tmp = start -> next;
  
  for (int i = 0; i < word.length(); i++ )
  {
    tmp -> add_state( word[i] );
    tmp = tmp -> next;
  }

  tmp -> add_state ( EPSILON );
  finish = tmp -> next;
  finish -> accept = true;

  //  finish -> command = word;
}

nfa::nfa( char a, char b )
{
  start = new state;
  finish = new state;
  start -> next = finish;
  finish -> accept = true;
  
  for (char tmp = a; tmp <= b; tmp++)
    start -> add_trans( finish, tmp );
}

nfa::nfa( ullong a, ullong b )
{
  start = new state;
  finish = new state;
  start -> next = finish;
  finish -> accept = true;
   
  for (ullong tmp = a; tmp <= b; tmp++)
    start -> add_trans( finish, tmp );
}

nfa::nfa()
{
  start = new state;
  finish = start;
  finish -> accept = false;
}

slist_node* slist::inlist( state* check )
{
  for (slist_node* tmp = head; tmp; tmp = tmp -> next)
    if ( tmp -> st == check ) return tmp;

  return NULL;
}

void slist::print()
{
  for (slist_node* tmp = head; tmp; tmp = tmp -> next)
    cout << tmp -> st -> index << " ";
  cout << endl;
}

void slist::insert( slist_node* toinsert )
{
  toinsert -> prev = NULL;
  toinsert -> next = head;
  head = toinsert;
}

void slist::remove( slist_node* toremove )
{
  slist_node* a;
  if (toremove == head)
    {
      a = head;
      head = head -> next;
      if (head)
	head -> prev = NULL;
      delete a;
    }
  else
    {// this seems overkill to remove... (also linear, not constant)
      for (slist_node* tmp = head; tmp; tmp = tmp -> next)
	if (tmp -> next == toremove)
	  {
	    a = tmp -> next;
	    tmp -> next = tmp -> next -> next;
	    if (tmp -> next)
	      tmp -> next -> prev = tmp;
	    delete a;
	  }
      
      /* don't know why this doesn't work...
       * if (toremove -> next)
       *   toremove -> next -> prev = toremove -> prev;
       * if (toremove -> prev)
       *   toremove -> prev -> next = toremove -> next; */
    }
}

void slist::do_epsilon()
{
  bool noeps = false;
  slist_node *tmp, *cur;
  
  while (!noeps) //while no new ones inserted
    { 
      noeps = true;
      for (tmp = head; tmp; tmp = tmp -> next) //for all in current list
	{
	  cur = tmp -> st -> do_trans(EPSILON); //do e transition
	  for (slist_node* l = cur; l; l = l -> next) 
	    if (!inlist(l -> st))
	      {
		slist_node* newnode = new slist_node(l -> st);
		insert( newnode );
		noeps = false;  //if you've inserted a new one, check again
	      }
	  slist_node* a;

	  for (slist_node* l = cur; l;) 
	    {
	      a = l -> next;
	      delete l;
	      l = a;
	    }
	}
    }
}

bool slist::do_letter(char let)
{
  slist_node *tmp, *tmp1, *cur;
  bool trans = false;
  
  for (tmp = head; tmp; tmp = tmp -> next) // for all in current list
    {
      if (cur = tmp->st->do_trans(let)) // if transition exists
	{
	  trans = true;
	  if (!tmp -> just_inserted) remove(tmp);
	  for (slist_node* l = cur; l; l = l -> next) // for all transitions
	    if (!(tmp1 = inlist (l->st)))  //if its not already in list...
	      {
		slist_node* newnode = new slist_node(l->st); //add it
		newnode -> just_inserted = true;
		insert( newnode );
	      }
	    else tmp1 -> just_inserted = true; //else dont delete it
	  
	  slist_node* a;
	  for (slist_node* l = cur; l;) 
	    {
	      a = l -> next;
	      delete l;
	      l = a;
	    }
	}
    
      else if (!tmp -> just_inserted) remove(tmp);
      //else remove
    }
  for (tmp = head; tmp; tmp = tmp -> next) 
    tmp -> just_inserted = false; 
  
  return trans;
}

bool slist::do_letter(ullong let)
{
  slist_node *tmp, *tmp1, *cur;
  bool trans = false;
  
  for (tmp = head; tmp; tmp = tmp -> next) // for all in current list
    {
      if (cur = tmp->st->do_trans(let)) // if transition exists
	{
	  trans = true;
	  if (!tmp -> just_inserted) remove(tmp);
	  for (slist_node* l = cur; l; l = l -> next) // for all transitions
	    if (!(tmp1 = inlist (l->st)))  //if its not already in list...
	      {
		slist_node* newnode = new slist_node(l->st); //add it
		newnode -> just_inserted = true;
		insert( newnode );
	      }
	    else tmp1 -> just_inserted = true; //else dont delete it
	  
	  slist_node* a;
	  for (slist_node* l = cur; l;) 
	    {
	      a = l -> next;
	      delete l;
	      l = a;
	    }
	}
      else if (!tmp -> just_inserted) remove(tmp);
      //else remove

    }
  for (tmp = head; tmp; tmp = tmp -> next) 
    tmp -> just_inserted = false; 
  
  return trans;
}

void slist::restart( state* st )
{
  slist_node* tmp, *tmp1;
  for (tmp = head; tmp;)
    {
      tmp1 = tmp -> next;
      delete tmp;
      tmp = tmp1;
    }

  delete tmp;
  head = new slist_node( st );
}

void nfa::take_input( conn& connection )
{
  index();
  slist states( start );
  char c;
  unsigned char in[8];
  bool accept = false;
  Display *disp = XOpenDisplay(NULL);
  Window root = RootWindow( disp, 0 );
  winlist windows;
  
  while (1)
    {
      states.do_epsilon();
      if (connection.Read(in) >= 0)
	{
	  ullong a = ullong(in[0]) | (ullong(in[1]) << 8)  |
	    (ullong(in[2]) << 16) | (ullong(in[3]) << 24) |
	    (ullong(in[4]) << 32) | (ullong(in[5]) << 40) |
	    (ullong(in[6]) << 48) | (ullong(in[7]) << 56);

	  //	  if (!(states.do_letter(a))) states.restart( start );
	  states.do_letter(a);
	  states.do_epsilon();
	  
	  windows.kill();
	  getwindows( windows, disp, root );
	  
	  //system commands, only run if window closed
	  //always run BEFORE other commands
	  for (slist_node* tmp = states.head; tmp; tmp = tmp -> next)
	    if ( tmp -> st -> accept
		 && tmp -> st -> what_to_do -> type == SYSTEM_ACTION)
	      {
		tmp -> st -> what_to_do -> do_event( windows, disp );
		accept = true;
	      }
	  
	  //everything else
	  for (slist_node* tmp = states.head; tmp; tmp = tmp -> next)
	    if ( tmp -> st -> accept
		 && tmp -> st -> what_to_do -> type != SYSTEM_ACTION)
	      {
		tmp -> st -> what_to_do -> do_event( windows, disp );
		accept = true;
	      }

	  if (accept)
	    {
	      states.restart( start );
	      accept = false;
	    }
	}
      else break;
    }
}

void nfa::addeps()
    {
  for (state* tmp = start -> next; tmp; tmp = tmp -> next)
    tmp -> add_trans( start, EPSILON );
}


bool nfa::accept_string( string s )
{
  bool retval = false;
  slist states( start );
  states.do_epsilon(); 
  for (int i = 0; i < s.length(); i++)
    {
      states.do_letter(s[i]);
      states.do_epsilon();
    }
  
  for (slist_node* tmp = states.head; tmp; tmp = tmp -> next)
    if ( tmp -> st -> accept ) retval = true;

  return retval;
}

//overloaded operators

nfa operator& (const nfa& n1, const nfa& n2 )
{
  nfa n3(n1);
  nfa* n4 = new nfa(n2);
  n3.finish -> next = n4 -> start;
  n3.finish -> add_trans( n4 -> start, EPSILON );
  n3.finish -> accept = false;
  n4 -> finish -> accept = true;
  n3.finish = n4 -> finish;
  n4 -> kill(2);
  delete n4;
  return n3;
}

nfa operator| ( const nfa& n1, const nfa& n2 )
{
  nfa n3(n1);
  nfa* n4 = new nfa(n2);
  n3.start -> add_trans( n4->start, EPSILON );
  n3.finish -> next = n4->start;
  n3.finish = n4->finish;
  n4 -> kill(2);
  delete n4;
  return n3;
}

nfa operator* ( const nfa& n1 )
{
  nfa n2(n1);
  n2.start -> add_trans ( n2.finish, EPSILON );
  n2.finish -> add_trans ( n2.start, EPSILON );
  return n2;
}

nfa operator+ ( const nfa& n1 )
{
  nfa n2(n1);
  n2.finish -> add_trans ( n2.start, EPSILON );
  return n2;
}

state* nfa::getindex(int ind)
{
  for (state* tmp = start; tmp; tmp = tmp -> next)
    if (tmp -> index == ind ) return tmp;
}

//copy constructor

nfa& nfa::operator=( const nfa& from )
{
  kill(0);
  from.index();
  
  start = new state;
  start -> accept = from.start -> accept;
  start -> index = from.start -> index;
  start -> next = NULL;
  start -> trans = NULL;
  start -> what_to_do = from.start -> what_to_do;
  state *p = start, *n;
  
  for (state* tmp = from.start -> next; tmp; tmp = tmp -> next)
    {
      n = new state;
      n -> accept = tmp -> accept;
      n -> index = tmp -> index;
      n -> next = NULL;
      n -> trans = NULL;
      n -> what_to_do = tmp -> what_to_do;
      p -> next = n;
      p = p -> next;
    }

  finish = n;

  state* tmp1 = from.start;
  transition *t1=NULL, *t2=NULL;
  for (state* tmp = start; tmp; tmp = tmp -> next)
    {
      for (transition* ttmp = tmp1 -> trans; ttmp; ttmp = ttmp -> next)
	{
	  t2 = new transition( getindex(ttmp->to->index), ttmp->input);
	  if (t1)
	    t1 -> next = t2;
	  else
	    tmp -> trans = t2;
	  t1 = t2;
	}
      tmp1 = tmp1 -> next;
      t1 = NULL; t2 = NULL;
    }
}

nfa::nfa( const nfa& from )
{
  from.index();
  
  start = NULL;
  state* p = NULL, *n=NULL;
  for (state* tmp = from.start; tmp; tmp = tmp -> next)
    {
      n = new state;
      n -> accept = tmp -> accept;
      n -> index = tmp -> index;
      n -> next = NULL;
      n -> trans = NULL;
      n -> what_to_do = tmp -> what_to_do;
      if (p)
	{
	  p -> next = n;
	  p = p -> next;
	}
      else
	{
	  start = n;
	  p = n;
	}
    }

  finish = n;

  state* tmp1 = from.start;
  transition *t1=NULL, *t2=NULL;
  for (state* tmp = start; tmp; tmp = tmp -> next)
    {
      for (transition* ttmp = tmp1 -> trans; ttmp; ttmp = ttmp -> next)
	{
	  t2 = new transition( getindex(ttmp->to->index), ttmp->input);
	  if (t1)
	    t1 -> next = t2;
	  else
	    tmp -> trans = t2;
	  t1 = t2;
	}
      tmp1 = tmp1 -> next;
      t1 = NULL; t2 = NULL;
    }
}

//destructor
void slist::kill()
{
  slist_node* tmp1, *tmp;
  for (tmp = head; tmp; )
    {
      tmp1 = tmp;
      tmp = tmp -> next;
      tmp1 -> next = NULL;
      delete tmp1;
    }
  delete tmp;
}

void nfa::kill( int method )
{
  if (method == 2)
    {
      start = NULL;
      finish = NULL;
    }
  
  state* stodel;
  transition* ttodel;
  for (state* tmp = start; tmp;)
    {
      for (transition* tmp1 = tmp -> trans; tmp1;)
	{
	  ttodel = tmp1;
	  tmp1 = tmp1 -> next;
	  delete ttodel;
	}
      stodel = tmp;
      tmp = tmp -> next;
      delete stodel;
    }

  if (method == 1)
    {
      start = new state;
      finish = new state;
      start -> next = finish;
      finish -> accept = true;
      start -> add_trans(finish, EPSILON);
    }
  else
    {
      start = NULL;
      finish = NULL;
    }
}
