/*
 * nfa.h - irx regexp matching class
 *
 * Copyright (C) 1998 Alan Penner        (apenner@andrew.cmu.edu)
 *                    Ryan de Jonckheere (rjd@andrew.cmu.edu)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 *
 */

#ifndef NFA_H
#define NFA_H

#include<iostream.h>
#include<string>
#include "irx.h"

#define EPSILON (ullong)0
#define ANY_LETTER (ullong)1

//structures

struct command;
struct conn;

struct nfa;
struct state;
struct transition;
struct slist;
struct slist_node;

struct nfa
{
  state* start;
  state* finish;

  nfa();
  nfa(string word);
  nfa(char a, char b);
  nfa( ullong a, ullong b );
  nfa( const nfa& from );
  nfa( ullong ircode );
  
  nfa& operator= (const nfa& n2);

  void print() const; //for debugging...
  void index() const;
  bool accept_string( string s );
  void take_input(conn& connection);
  void addeps();
  void kill( int method );
  ~nfa() { kill(0); };
  
  state* getindex(int ind);

};

struct state
{
  bool accept;
  transition* trans;
  state* next;
  int index;
  command* what_to_do;
  
  state(): accept(false), trans(NULL), next(NULL), what_to_do(NULL) {};
  
  void add_trans( state* to, char let );
  void add_trans( state* to, ullong ircode );
  void add_state( char let );
  void add_state( ullong ircode );
  slist_node* do_trans( char let );
  slist_node* do_trans( ullong let ); 
};

struct transition
{
  //  char input;
  ullong input;
  state* to;
  transition* next;
  
  transition(state* t, char let): to(t), input(let), next(NULL) {};
  transition(state* t, ullong let): to(t), input(let), next(NULL) {};
};

struct slist_node
{
  state* st;
  slist_node* prev;
  slist_node* next;
  bool just_inserted;
  
  slist_node(state* s): st(s), next(NULL), prev(NULL), just_inserted(false) {};
};

struct slist
{
  slist_node* head;
  
  slist(state* s): head(new slist_node(s)) {};
  ~slist() { kill(); };

  slist_node* inlist( state* check );
    
  void insert( slist_node* toinsert );
  void remove( slist_node* toremove );
  void do_epsilon();
  bool do_letter(char let);
  bool do_letter(ullong let);
  void print();
  void restart( state* s );
  void kill();
};

//overloaded operator prototypes

nfa operator& (const nfa& n1, const nfa& n2);
nfa operator| (const nfa& n1, const nfa& n2);
nfa operator* (const nfa& n1);
nfa operator+ (const nfa& n1);

#endif

